//                                               -*- C++ -*-
/**
 *  @file  Beta.hxx
 *  @brief The Beta distribution
 *
 *  (C) Copyright 2005-2007 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2009-05-28 14:47:53 +0200 (jeu. 28 mai 2009) $
 *  Id:      $Id: Beta.hxx 1262 2009-05-28 12:47:53Z dutka $
 */
#ifndef OPENTURNS_BETA_HXX
#define OPENTURNS_BETA_HXX

#include "NonEllipticalDistribution.hxx"

namespace OpenTURNS {

  namespace Uncertainty {

    namespace Distribution {


      /**
       * @class Beta
       *
       * The Beta distribution.
       */
      class Beta
        : public Model::NonEllipticalDistribution
      {
        CLASSNAME;
      public:

        typedef Base::Common::InvalidArgumentException               InvalidArgumentException;
        typedef Model::NonEllipticalDistribution                     NonEllipticalDistribution;  // required by SWIG
        typedef NonEllipticalDistribution::NumericalPoint            NumericalPoint;
        typedef NonEllipticalDistribution::NumericalSample           NumericalSample;
        typedef NonEllipticalDistribution::CovarianceMatrix          CovarianceMatrix;
        typedef NonEllipticalDistribution::NumericalPointWithDescriptionCollection  NumericalPointWithDescriptionCollection;
        typedef NonEllipticalDistribution::NotDefinedException       NotDefinedException;
        typedef NonEllipticalDistribution::StorageManager            StorageManager;


        enum ParameterSet { RT, MUSIGMA };


        /** Default constructor */
        Beta();

        /** Parameters constructor */
        Beta(const NumericalScalar arg1,
             const NumericalScalar arg2,
             const NumericalScalar a,
             const NumericalScalar b,
             const ParameterSet set = RT)
          throw (InvalidArgumentException);

        /** Comparison operator */
        Bool operator ==(const Beta & other) const;

        /** String converter */
	String __repr__() const;



        /* Interface inherited from Distribution */

        /** Virtual constructor */
        Beta * clone() const;

        /** Get one realization of the distribution */
        NumericalPoint getRealization() const;

        /** Get the DDF of the distribution */
        using NonEllipticalDistribution::computeDDF;
        NumericalPoint computeDDF(const NumericalPoint & point) const;

        /** Get the PDF of the distribution */
        using NonEllipticalDistribution::computePDF;
        NumericalScalar computePDF(const NumericalPoint & point) const;

        /** Get the CDF of the distribution */
        using NonEllipticalDistribution::computeCDF;
        NumericalScalar computeCDF(const NumericalPoint & point, const Bool tail = false) const;

        /** Get the PDFGradient of the distribution */
        NumericalPoint computePDFGradient(const NumericalPoint & point) const;

        /** Get the CDFGradient of the distribution */
        NumericalPoint computeCDFGradient(const NumericalPoint & point) const;

        /** Get the quantile of the distribution */
        NumericalScalar computeScalarQuantile(const NumericalScalar prob,
                                              const NumericalScalar initialGuess = 0.0,
                                              const NumericalScalar initialStep = 1.0,
                                              const NumericalScalar precision = DefaultQuantileEpsilon) const;

        /** Get the roughness, i.e. the L2-norm of the PDF */
        NumericalScalar getRoughness() const;

        /** Get the mean of the distribution */
        NumericalPoint getMean() const throw(NotDefinedException);

        /** Get the standard deviation of the distribution */
        NumericalPoint getStandardDeviation() const throw(NotDefinedException);

        /** Get the skewness of the distribution */
        NumericalPoint getSkewness() const throw(NotDefinedException);

        /** Get the kurtosis of the distribution */
        NumericalPoint getKurtosis() const throw(NotDefinedException);

        /** Get the covariance of the distribution */
        CovarianceMatrix getCovariance() const throw(NotDefinedException);

        /** Parameters value and description accessor */
        NumericalPointWithDescriptionCollection getParametersCollection() const;
        using NonEllipticalDistribution::setParametersCollection;
        void setParametersCollection(const NumericalPointCollection & parametersCollection);

        /** Check if the distribution is elliptical */
        Bool isElliptical() const;


        /* Interface specific to Beta */

        /** R accessor */
        void setR(const NumericalScalar r)
          throw(InvalidArgumentException);
        NumericalScalar getR() const;

        /** T accessor */
        void setT(const NumericalScalar t)
          throw(InvalidArgumentException);
        NumericalScalar getT() const;

        /** Mu accessor */
        void setMuSigma(const NumericalScalar mu,
                        const NumericalScalar sigma);
        NumericalScalar getMu() const;

        /** Sigma accessor */
        NumericalScalar getSigma() const;

        /** A accessor */
        void setA(const NumericalScalar a);
        NumericalScalar getA() const;

        /** B accessor */
        void setB(const NumericalScalar b);
        NumericalScalar getB() const;

        /** Method save() stores the object through the StorageManager */
        void save(const StorageManager::Advocate & adv) const;

        /** Method load() reloads the object from the StorageManager */
        void load(const StorageManager::Advocate & adv);


      protected:


      private:

        /** RT accessor that avoid a check between the setting of r and the setting of t */
        void setRT(const NumericalScalar r,
                   const NumericalScalar t)
          throw(InvalidArgumentException);

        /** Compute the numerical range of the distribution given the parameters values */
        void computeRange();

        /** Update the derivative attributes */
        void update();

        /** The main parameter set of the distribution */
        NumericalScalar r_;
        NumericalScalar t_;
        NumericalScalar a_;
        NumericalScalar b_;
        NumericalScalar normalizationFactor_;
      }; /* class Beta */


    } /* namespace Distribution */
  } /* namespace Uncertainty */
} /* namespace OpenTURNS */

#endif /* OPENTURNS_BETA_HXX */
