//                                               -*- C++ -*-
/**
 *  @file  LinearCombinationEvaluationImplementation.hxx
 *  @brief The evaluation part of linear combination of polynomials
 *
 *  (C) Copyright 2005-2007 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2008-05-21 17:44:02 +0200 (Wed, 21 May 2008) $
 *  Id:      $Id: Object.hxx 818 2008-05-21 15:44:02Z dutka $
 */
#ifndef OPENTURNS_LINEARCOMBINATIONEVALUATIONIMPLEMENTATION_HXX
#define OPENTURNS_LINEARCOMBINATIONEVALUATIONIMPLEMENTATION_HXX

#include "NumericalMathEvaluationImplementation.hxx"
#include "Collection.hxx"
#include "NumericalMathFunction.hxx"

namespace OpenTURNS
{

  namespace Base
  {

    namespace Func
    {

      /**
       * @class LinearCombinationEvaluationImplementation
       *
       * The evaluation part of linear combination of polynomials
       */

      class LinearCombinationEvaluationImplementation
	: public NumericalMathEvaluationImplementation
      {
	CLASSNAME;
      public:

	typedef Base::Common::StorageManager                                         StorageManager;
	typedef NumericalMathEvaluationImplementation::NumericalPoint                NumericalPoint;
	typedef NumericalMathEvaluationImplementation::NumericalPointWithDescription NumericalPointWithDescription;
	typedef NumericalMathEvaluationImplementation::Matrix                        Matrix;
	typedef NumericalMathEvaluationImplementation::InvalidArgumentException      InvalidArgumentException;
	typedef NumericalMathEvaluationImplementation::InternalException             InternalException;
	typedef Type::Collection<NumericalMathFunction>                              NumericalMathFunctionCollection;

	// friend class Base::Common::Factory<LinearCombinationEvaluationImplementation>;

	/** Default constructor */
	LinearCombinationEvaluationImplementation();

      public:
	/** Parameter constructor */
	LinearCombinationEvaluationImplementation(const NumericalMathFunctionCollection & functionsCollection,
						  const NumericalPoint & coefficients);

	/** Virtual constructor */
	virtual LinearCombinationEvaluationImplementation * clone() const;

	/** String converter */
	virtual String __repr__() const;

	/** Evaluation operator */
	NumericalPoint operator () (const NumericalPoint & in) const
          throw(InvalidArgumentException, InternalException);

	/** Coefficients accesso */
	NumericalPoint getCoefficients() const;

	/** Functions accessor */
	NumericalMathFunctionCollection getFunctionsCollection() const;
	void setFunctionsCollectionAndCoefficients(const NumericalMathFunctionCollection & functionsCollection,
						   const NumericalPoint & coefficients);

	/** Input dimension accessor */
	UnsignedLong getInputNumericalPointDimension() const
          throw(InternalException);

	/** Output dimension accessor */
	UnsignedLong getOutputNumericalPointDimension() const
          throw(InternalException);

        /** Gradient according to the marginal parameters */
        Matrix parametersGradient(const NumericalPoint & in) const;

        /** Parameters value and description accessor */
        NumericalPointWithDescription getParameters() const;

	/** Method save() stores the object through the StorageManager */
	virtual void save(const StorageManager::Advocate & adv) const;

	/** Method load() reloads the object from the StorageManager */
	virtual void load(const StorageManager::Advocate & adv);


      protected:

      private:
	// Make the gradient and the hession friend classes of the evaluation in order to share the functions and the coefficients
	friend class LinearCombinationGradientImplementation;
	friend class LinearCombinationHessianImplementation;

	// The functions of the combination
	NumericalMathFunctionCollection functionsCollection_;

	// The coefficients of the combination
	NumericalPoint coefficients_;
      } ; /* class LinearCombinationEvaluationImplementation */


    } /* namespace Func */
  } /* namespace Base */
} /* namespace OpenTURNS */

#endif /* OPENTURNS_LINEARCOMBINATIONEVALUATIONIMPLEMENTATION_HXX */
