//                                               -*- C++ -*-
/**
 * @file  LinearNumericalMathEvaluationImplementation.hxx
 * @brief Class for a linear numerical math function implementation
 *        of the form y = constant + <linear, x - c> where c is a
 *        dim(x) numerical point, linear a dim(x) by dim(y) matrix
 *        and  <linear, x - c> means Transpose(linear).(x - c)
 *
 * (C) Copyright 2005-2006 EADS
 *
 * Permission to copy, use, modify, sell and distribute this software
 * is granted provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied
 * warranty, and with no claim as to its suitability for any purpose.
 *
 *
 * \author $LastChangedBy: dutka $
 * \date   $LastChangedDate: 2009-05-28 14:47:53 +0200 (jeu. 28 mai 2009) $
 */

#ifndef OPENTURNS_LINEARNUMERICALMATHFUNCTIONIMPLEMENTATION_HXX
#define OPENTURNS_LINEARNUMERICALMATHFUNCTIONIMPLEMENTATION_HXX

#include "NumericalMathEvaluationImplementation.hxx"
#include "Exception.hxx"
#include "NumericalPoint.hxx"
#include "NumericalSample.hxx"
#include "Matrix.hxx"

namespace OpenTURNS {

  namespace Base {

    namespace Func {


      /**
       * @class LinearNumericalMathEvaluationImplementation
       *
       * This class offers an interface for an real numerical mathematical function
       * implementations into the platform. It realizes the computation of the image vector
       * of a vector (aka NumericalPoint) through the function. The
       * function may be loaded in an external way through a wrapper.
       */
      class LinearNumericalMathEvaluationImplementation
	: public NumericalMathEvaluationImplementation
      {
	CLASSNAME;
      public:

	typedef NumericalMathEvaluationImplementation::NumericalPoint  NumericalPoint;
	typedef NumericalMathEvaluationImplementation::Matrix          Matrix;
	typedef NumericalMathEvaluationImplementation::NumericalSample NumericalSample;
	typedef NumericalMathEvaluationImplementation::InvalidArgumentException InvalidArgumentException;
	typedef NumericalMathEvaluationImplementation::InternalException InternalException;
	typedef NumericalMathEvaluationImplementation::StorageManager            StorageManager;
	typedef Common::InvalidDimensionException InvalidDimensionException;

	/** Default constructor */
	LinearNumericalMathEvaluationImplementation();

	/** Parameter constructor */
	LinearNumericalMathEvaluationImplementation(const NumericalPoint & center,
						    const NumericalPoint & constant,
						    const Matrix & linear) throw(InvalidDimensionException);


	/** Virtual constructor */
	virtual LinearNumericalMathEvaluationImplementation * clone() const;

	/** Comparison operator */
	Bool operator ==(const LinearNumericalMathEvaluationImplementation & other) const;

	/** String converter */
	virtual String __repr__() const;



	/* Here is the interface that all derived class must implement */
	
	/** Operator () */
	virtual NumericalPoint operator() (const NumericalPoint & in) const 
          throw(InvalidArgumentException, InternalException);

	/** Operator () */
	virtual NumericalSample operator() (const NumericalSample & in) const 
          throw(InvalidArgumentException, InternalException);

	/** Accessor for input point dimension */
	virtual UnsignedLong getInputNumericalPointDimension() const
          throw(InternalException);

	/** Accessor for output point dimension */
	virtual UnsignedLong getOutputNumericalPointDimension() const
	  throw(InternalException);

	/** Accessor for the center */
	NumericalPoint getCenter() const;

	/** Accessor for the constant term */
	NumericalPoint getConstant() const;

	/** Accessor for the linear term */
	Matrix getLinear() const;

 	/** Method save() stores the object through the StorageManager */
	void save(const StorageManager::Advocate & adv) const;

	/** Method load() reloads the object from the StorageManager */
	void load(const StorageManager::Advocate & adv);

      protected:

    
      private:
	NumericalPoint center_;
	NumericalPoint constant_;
	Matrix linear_;
      }; /* class LinearNumericalMathEvaluationImplementation */


    } /* namespace Func */
  } /* namespace Base */
} /* namespace OpenTURNS */

#endif /* OPENTURNS_LINEARNUMERICALMATHFUNCTIONIMPLEMENTATION_HXX */
