//                                               -*- C++ -*-
/**
 *  @file  TensorImplementation.hxx
 *  @brief TensorImplementation implements the Tensor classes
 *
 *  (C) Copyright 2005-2007 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2009-05-28 14:47:53 +0200 (jeu. 28 mai 2009) $
 *  Id:      $Id: TensorImplementation.hxx 1262 2009-05-28 12:47:53Z dutka $
 */
#ifndef OPENTURNS_TENSORIMPLEMENTATION_HXX
#define OPENTURNS_TENSORIMPLEMENTATION_HXX

#include "PersistentCollection.hxx"
#include "NumericalPoint.hxx"
#include "Exception.hxx"
#include "Matrix.hxx"
#include "SymmetricMatrix.hxx"

namespace OpenTURNS
{

  namespace Base
  {

    namespace Type
    {

      /**
       * @class TensorImplementation
       *
       * TensorImplementation implements the Tensor classes
       */

      class TensorImplementation
	: public PersistentCollection<NumericalScalar>
      
      {
	CLASSNAME;
	
      public:
	
	typedef Common::InvalidDimensionException InvalidDimensionException ;
	typedef Common::InvalidArgumentException  InvalidArgumentException ;
	
	/** Default constructor */
	TensorImplementation();

	/** Constructor with size (rowDim, colDim, sheetDim) */
	TensorImplementation(const UnsignedLong rowDim,
			     const UnsignedLong colDim,
			     const UnsignedLong sheetDim);
	
	/** Constructor from range of external collection */
	template <class InputIterator>
	TensorImplementation(const UnsignedLong rowDim,
			     const UnsignedLong colDim,
			     const UnsignedLong sheetDim,
			     InputIterator first,
			     InputIterator last);
	       
	/** Constructor from external collection */
	/** If the dimensions don't correspond with the size of the collection, */
	/** the collection is either truncated or completed with zeros*/
	TensorImplementation(const UnsignedLong rowDim,
			     const UnsignedLong colDim,
			     const UnsignedLong sheetDim,
			     const Collection<NumericalScalar> & elementsValues);
	

	/** Virtual constructor */
	virtual TensorImplementation * clone() const;

	/** String converter */
	virtual String __repr__() const;
	virtual String __str__() const;
	
	/** Operator () gives access to the elements of the tensor (to modify these elements) */
	/** The element of the tensor is designated by its row number i, its column number j and its sheet number k */
	NumericalScalar & operator () (const UnsignedLong i,
				       const UnsignedLong j,
				       const UnsignedLong k) throw(InvalidDimensionException);
	
	/** Operator () gives access to the elements of the tensor (read only) */
	/** The element of the tensor is designated by its row number i, its column number j and its sheet number k */
	const NumericalScalar & operator () (const UnsignedLong i,
					     const UnsignedLong j,
					     const UnsignedLong k) const throw(InvalidDimensionException);
	
	/** getSheet returns the sheet specified by its sheet number k */
	Matrix getSheet(const UnsignedLong k) const throw(InvalidDimensionException);
	
	/** setSheet sets matrix m as the sheet specified by its sheet number k  */
	void setSheet(const UnsignedLong k,
		      const Matrix & m) throw(InvalidDimensionException);
	
	/** getSheetSym returns the symmetric sheet specified by its sheet number k */
	SymmetricMatrix getSheetSym(const UnsignedLong k) const throw(InvalidDimensionException);
	
	/** setSheetSym sets symmetric matrix m as the sheet specified by its sheet number k  */
	void setSheetSym(const UnsignedLong k,
			 const SymmetricMatrix & m) throw(InvalidDimensionException, InvalidArgumentException);
	
	/** Get the dimensions of the tensor */
	/** Number of rows */
	const UnsignedLong getNbRows() const ;
	/** Number of columns */
	const UnsignedLong getNbColumns() const ;
	/** Number of sheets */
	const UnsignedLong getNbSheets() const ;
		
	/** Symmetrize TensorImplementation in case it is a symmetric tensor (stored as a set of triangular matrix sheets) */
	void symmetrize() const;
	
	/** Comparison operators */
	Bool operator == (const TensorImplementation & rhs) const ;
	
	/** Empty returns true if there is no element in the tensor */
	const Bool isEmpty() const ;

 	/** Method save() stores the object through the StorageManager */
	void save(const StorageManager::Advocate & adv) const;

	/** Method load() reloads the object from the StorageManager */
	void load(const StorageManager::Advocate & adv);

      protected:
        
	/** TensorImplementation Dimensions */
	UnsignedLong nbRows_;
	UnsignedLong nbColumns_;
	UnsignedLong nbSheets_;
	
        /** Position conversion function : the indices i & j are used to compute the actual position of the element in the collection */
	inline UnsignedLong convertPosition (const UnsignedLong i,
					     const UnsignedLong j,
					     const UnsignedLong k) const ;

      }; /* class TensorImplementation */

      /** Constructor from range of external collection */
      template <class InputIterator>
      TensorImplementation::TensorImplementation(const UnsignedLong rowDim,
						 const UnsignedLong colDim,
						 const UnsignedLong sheetDim,
						 InputIterator first,
						 InputIterator last)
	: PersistentCollection<NumericalScalar>(rowDim * colDim * sheetDim),
	  nbRows_(rowDim),
	  nbColumns_(colDim),
	  nbSheets_(sheetDim)
      {
	this->assign(first, last);
      }
      
      /** Inline functions */
      
      inline UnsignedLong TensorImplementation::convertPosition (const UnsignedLong i,
								 const UnsignedLong j,
								 const UnsignedLong k) const
      {
        return i + nbRows_ * (j + nbColumns_ * k) ;
      }

    } /* namespace Common */
  } /* namespace Base */
} /* namespace OpenTURNS */

#endif /* OPENTURNS_TENSORIMPLEMENTATION_HXX */
